;LTC6946-Switched.ASM     G4JNT  July 2013
;GPIO3 pulled high at start goes into normal operation (needs resistor, no weak pullup)
;GPIO3 low = programming mode


;  Serial Command  protocol     
;	Px		Page 0 - 3, complete register set for each frequency
;	Rxxxx		16 bit ASCII data, terminated with [cr]  
;	W               Write the data just sent to the appropriate register defined by first two hex characters
;	EXIT		Resume normal operation

  

    PROCESSOR 12F629
    INCLUDE "P12F629.INC"
    errorlevel 1



;   ===============================================================

    #define     SerIn       GPIO, 0     ;Also PGD,  Port change enabled  Pin 7 and Frq sel link
    #define     SerOut      GPIO, 1     ;Also PGC			 Pin 6  "  "    "   "
    #define     Data   	    GPIO, 2     ;Pin 5
    #define     Clk   	    GPIO, 4     ;Pin 3
    #define     LE	    GPIO, 5     ;Pin 2

    #define	ContMode	Flags, 0
    #define	ProgFlag	Flags, 1

    #define     LED         GPIO, 1     ;LED on SerOut for debugging

 RS232POL   =   1         ; 1 = RS232 levels,  0 = Logic/FTDI Interface
 BAUDCONST  =   d'14'      ;6.N + 18  =  1000000/Baud   Half baud delay, 9600 baud

  __config  0x3FC4      ;INTOSC, I/O on GP4/5
  ;CPD Off, CP Off, Boden On, MCLRE = I/O, PWRTE on, WDT Off,

  __idlocs  0x806C



  org	0x2100

		;8 bit words sent in sequence for each of the four frequencies  

ConfigData			;	10MHzFref, 0.5MHz Fcomp, divider 6, 550/540/530/520MHz out
;        LTC6946 Startup Settings sent in this order
	de 	0x38		;Reg 1	Status Mask	  	
	de	0x08		;2	Power reg		On Immediately
	de	0x00, 0x05	;3/4	B and R registers	R = 5
	de	0x06, 0x72	;5/6	N divider		N = 1650 0x0672
	de	0x63		;7	ALC and calibration
	de	0xFE		;8	Ref input and O/P divider
	de	0xCF		;9	LDet and Charge pump current	2mA
	de	0xC0		;A	Charge Pump function
	de	0,0,0,0,0,0	;Pad out to 16 bytes for easy addressing

	de 	0x38		;	
	de	0x08		;	
	de	0x00, 0x05	;			
	de	0x06, 0x54	;	
	de	0x63		;
	de	0xFE		;
	de	0xCF		;
	de	0xC0		;
	de	0,0,0,0,0,0	;
			
	de 	0x38		;	  	
	de	0x08		;	
	de	0x00, 0x05	;			
	de	0x06, 0x36	;	
	de	0x63		;
	de	0xFE		;
	de	0xCF		;
	de	0xC0		;
	de	0,0,0,0,0,0	;

	de 	0x38		;
	de	0x08		;
	de	0x00, 0x05	;		
	de	0x06, 0x18	;
	de	0x63		;
	de	0xFE		;
	de	0xCF		;	
	de	0xC0		;
	de	0,0,0,0,0,0	;

;==================	 
  org   0
    nop
    clrw
    movwf   INTCON          ;disable interupts
    goto    StartUp         ;jump to main code
;================================
  org 4     ;Interrupt routine used for serial I/O
    movwf   Wreg            ;Save W
    swapf   STATUS , W      ; nb.  movwf does not affect any STATUS bits
    movwf   StsReg          ;Stave STATUS, without affecting it

    btfss   INTCON , GPIF   ;Test for port B change
    goto    CarryOnInt      ;
    bcf     ContMode
                            ;
    bcf     INTCON , GPIE   ;
    call    Assemble232     ;MUSTN'T Waste ANY TIME UP TO HERE !!!!

    sublw   0x0D            ;Check for [cr] terminating any command,
    btfsc   STATUS, Z       ;  otherwise chars are pushed onto stack
    call    CheckSerCmd     ;Decode the command and do any actions

    bcf     INTCON , GPIF   ;Clear the flag
    bsf     INTCON , GPIE   ;Re-enable the interrupt
;...............
CarryOnInt
    swapf   StsReg , W      ;Restore STATUS
    movwf   STATUS
    swapf   Wreg            ;Restore W
    swapf   Wreg , W        ; nb.  movf affects Z bit

    retfie
;=================================
StartUp 
    	movlw   7
    	movwf   CMCON
    	bsf     STATUS,RP0      ;ram page 1

    	movlw   b'00001011'     ;For startup GPIO0/1/3 are inputs 
    	movwf   TRISIO          ;
	movwf	WPU		;Enable weak pull ups on GP1/0 for links

    	movlw   b'00000000'     ;Pull-ups enabled
	movwf	OPTION_REG

    	call    0x3FF           ;Get factory stored OSCCAL value
    	movwf   OSCCAL
    	bcf     STATUS, RP0

	bsf	LE

	clrf	Flags
	call	ClearStack

	btfss	GPIO, 3		;Pulled low for programming
	goto	ProgMode

RetFromProg
	call	ReadLinks 	;Get and load start up values
	movwf	PageOffset
	call	SendFromEE
	call	LongDelay
	call	LongDelay
	call	SendFromEE	;Send it all again to make sure


MainLoop        ;		;All interrupt driven from now on
	call	LongDelay
	call	ReadLinks	
	xorwf	PageOffset, W
	btfsc	STATUS, Z
	goto	MainLoop	;If no change, do nothing
	
	call	ReadLinks	;Recover the new value
	movwf	PageOffset	;Update with new frequency
	call	SendFromEE	

    	goto    MainLoop
;................................
ProgMode
	bsf	ProgFlag	
    	bsf     STATUS,RP0      ;
    	movlw   b'00001001'     ;GPIO,1 output for SerOut
    	movwf   TRISIO          ;
	movlw	1
	movwf   IOC             ;Port change interrupt on GP0
	bcf	STATUS, RP0

  if RS232POL == 1
    	bcf   SerOut      ;RS232 idle state (note RS232 polarity !)
  else
    	bsf   SerOut
  endif

	call	IntroMsg
	call	ShowAll


    	movlw   b'10001000'      ;Port Change interrupt
    	movwf   INTCON

ProgLoop
	btfsc	ProgFlag	;Cleared after 'Init' command
	goto	ProgLoop	;All prog mode is interrupt driven

    	bsf     STATUS,RP0      ;
    	movlw   b'00001011'     ;Back to inputs for link selection
   	movwf   TRISIO       
	clrf	IOC
	bcf	STATUS, RP0
	clrf	INTCON	

	goto	RetFromProg
;======================================

ReadLinks
	comf	GPIO, W		;Link installed = '1'
	andlw	0x03
	movwf	Temp
	swapf	Temp, W		;Multiply link value, 0 - 3, by 16 
	return			;Return with new page only in W, to allow testing in the main loop
;-------------------------------
SendFromEE			;Send entire register set to synth chip and echo
	clrf	RegCount
	bcf	LE		;Use the auto increment multiple block write 
	movlw	2
	call	SPIByte		;Address 1 left justified with W bit for start of memory

SendEELoop			;Use EndConfigData to terminate	
	movf	RegCount, W	;RegCount goes from 0 to 9
	movwf	D1		;Ready for echoing back
	incf	D1		;As registers start at 1

	addlw	LOW(ConfigData)
	addwf	PageOffset, W
	call	GetEE_12F
	movwf	D0
	call	SPIByte	

	incf	RegCount	
	movlw	d'10'

	subwf	RegCount, W		;W = Address - EndOfDataLabel, when 0/+Ve, C=1, exit sending
	btfss	STATUS, C	
	goto	SendEELoop

	bsf	LE		

	return
;------------------------------------
SPIByte                ;Data in W, sent on SPI bus, assuming LE already set
	movwf	Temp
	movwf	Temp2	; Only used for DEBUG below

	movlw	d'8'
	movwf	BitCount
SendChipLoop
	bcf	Clk
	rlf	Temp		;Working bit in C
	btfsc	STATUS, C	
	bsf	Data		
	btfss	STATUS, C
	bcf	Data
	nop
	nop
	bsf	Clk		;Clocked in on rising edge
	decfsz	BitCount
	goto	SendChipLoop

	bcf	Clk		;Place into rest state
	bcf	Data

	return

;------------------------------------
ShowAll				;Echo back all four pages
	clrf	PageOffset
ShowallLoop
	movlw	"P"
	call	SendByte
	movlw	"a"
	call	SendByte
	movlw	"g"
	call	SendByte
	movlw	"e"
	call	SendByte
	movlw	" "
	call	SendByte
	swapf	PageOffset, W
	addlw	0x30
	call	SendByte	
	movlw	" "
	call	SendByte
	call	CrLf
	
	call	ShowBlock
	
	movlw	0x10
	addwf	PageOffset
	btfss	PageOffset, 6	;Overflow equivalent to page 4
	goto	ShowallLoop
	
	return
;-------------------------------
ShowBlock
	clrf	RegCount
ShowEELoop				
	movf	RegCount, W	;RegCount goes from 0 to 9
	movwf	D1		;Ready for echoing back
	incf	D1		;As registers start at 1

	addlw	LOW(ConfigData)
	addwf	PageOffset, W
	call	GetEE_12F
	movwf	D0
	call	SendBack	;Sends D1, D0

	incf	RegCount	
	movlw	d'10'

	subwf	RegCount, W		
	btfss	STATUS, C	
	goto	ShowEELoop

	bsf	LE		

	return
	return
;-------------------------------
SendBack              
	movf	D1, W		;D1 contains address, D0 contents
	movwf	X1
	movf	D0, W
	movwf	X0

	movlw	d'16'
	movwf	BitCount
SendBackLoop
	rlf	X0
	rlf	X1
	movlw	"0"
	btfsc	STATUS, C
	movlw	"1"
	call	SendByte	

	movf	BitCount, W
	andlw	0x03
	sublw	0x01
	btfss	STATUS, Z
	goto	NoSp
	movlw	" "
	call	SendByte
NoSp
	decfsz	BitCount
	goto	SendBackLoop

	movlw	" "
	call	SendByte
	movlw	" "
	call	SendByte		;Now as hex
	movlw	"0"
	call	SendByte
	movlw	"x"
	call	SendByte

	swapf	D1, W
	call	SendHex		
	movf	D1, W
	call	SendHex
	movlw	"-"
	call	SendByte
	swapf	D0, W
	call	SendHex		
	movf	D0, W
	call	SendHex
	call	CrLf

	return
;----------------------------------------------
DecodeHex           ;Single ASCII character to binary 
    movwf   Temp            ;Uses Temp
    movlw   0x30
    subwf   Temp            ;subtract $30 and leave in Temp
    btfss   STATUS , C      ;if < $30, result -ve so C=0, error, set=0
    clrf    Temp

    movf    Temp , W        
    sublw   d'9'            ;W = 9 - W
    btfsc   STATUS , C      ;if -ve, C=0, char > 9, so subtract 7
    goto    ConvDone
    movlw   d'7'
    subwf   Temp            ;
ConvDone
    movf    Temp , W        ;
    andlw   0x0F            ;mask out any rubbish left in
    return              
;------------------------------------
SendHex         ;Send a single Nibble as RS232
    andlw   0x0F
    addlw   0x30
    movwf   Temp
    sublw   0x39
    btfsc   STATUS , C
    goto    CharDone
    movlw   7
    addwf   Temp
CharDone
    movf    Temp , W
    call    SendByte
    return
;------------------------------
CrLf
    movlw   0x0D
    call    SendByte
    movlw   0x0A
    call    SendByte
    return
;---------------------------------
SendByte                    
	movwf   Temp
 if RS232POL == 1
    	bsf     SerOut
 else
    	bcf     SerOut             ;start bit
 endif
	call    HalfBitDelay        ;
	call    HalfBitDelay
	movlw   8
	movwf   Counter
SendDatLoop
	rrf     Temp                ;bit into C

 if RS232POL == 1
	btfsc   STATUS , C
    	bcf     SerOut
	btfss   STATUS , C
    	bsf     SerOut             ;could have 2us jitter depending on 1/0
 else
	btfsc   STATUS , C
    	bsf     SerOut
	btfss   STATUS , C
    	bcf     SerOut             ;could have 2us jitter depending on 1/0
 endif

	call    HalfBitDelay
	call    HalfBitDelay
	decfsz  Counter
	goto    SendDatLoop         ;loop 8 + BitDelay long

 if RS232POL == 1
    	bcf     SerOut
 else
    	bsf     SerOut
 endif
	call    HalfBitDelay    
	call    HalfBitDelay        ;Two stop bits
	call    HalfBitDelay
	call    HalfBitDelay

	return
;-------------------------------
CheckSerCmd             ;[cr] terminator has been received, so decode
                        ; the commands in here. The [cr]  Will be in Stack1
	clrf	D0
	clrf	D1


	movlw   "R"         ;		4 characters
    	subwf   Stack6, W   ;Stack 6/5/4/3/2/1
    	btfsc   STATUS, Z   ;      R x x x x [cr]
    	goto    RCmd  

	movlw   "R"         ;		3 characters, as no address > 0x0A
    	subwf   Stack5, W   ;Stack 5/4/3/2/1
    	btfsc   STATUS, Z   ;      R x x x [cr]
    	goto    RCmdShort 
 
	movlw   "E"         ;
    	subwf   Stack5, W   ;Stack 5/4/3/2/1
    	btfsc   STATUS, Z   ;      E X I T [cr]
    	goto    ECmd  

	movlw   "W"         ;
    	subwf   Stack2, W   ;Stack 5/4/3/2/1
    	btfsc   STATUS, Z   ;            W[cr]
    	goto    WCmd  

	movlw   "P"         ;
    	subwf   Stack3, W   ;Stack 5/4/3/2/1
    	btfsc   STATUS, Z   ;          P n [cr]
    	goto    PCmd  
    	goto    DoneCmd

;.............
RCmd		;Extract  hex data and left align into D1/0
	movf	Stack5, W
	call	DecodeHex
	movwf	D1
	swapf	D1
RCmdShort
	movf	Stack4, W
	call	DecodeHex
	addwf	D1		;D1 contains address, D0 contents

	movf	Stack3, W
	call	DecodeHex
	movwf	D0
	swapf	D0
	movf	Stack2, W
	call	DecodeHex
	addwf	D0

	call	SendBack	;D1/D0 to terminal shown as bits and hex 

	bcf	LE		;Send this single byte to synth chip
	bcf	STATUS, C
	rlf	D1, W		;B0 = Write bit, address left justified
	call	SPIByte		;Send address
	movf	D0, W
	call	SPIByte		;Send the sdata
	bsf	LE

	movf	D1, W		;Make a backup, used for writing to EE
	movwf	X1
	movf	D0, W
	movwf	X0
	goto    DoneCmd
;.............

;.............
ECmd			;INIT,  sends all data from EE
	movlw	"X"
	subwf	Stack4
	btfss	STATUS, Z
	goto	DoneCmd		;Get out if INIT has not been entered in full 

	movlw	"I"
	subwf	Stack3
	btfss	STATUS, Z
	goto	DoneCmd	

	movlw	"T"
	subwf	Stack2
	btfss	STATUS, Z
	goto	DoneCmd	

	call	ShowAll

	call	CrLf
	movlw	"E"
	call	SendByte
	movlw	"x"
	call	SendByte
	movlw	"i"
	call	SendByte
	movlw	"t"
	call	SendByte
	call	CrLf


	bcf	ProgFlag	;Causes exit from programming mode
	goto	DoneCmd
;..............
WCmd				;Writes data just entered to the appropriate EE location 
				;  Uses the backup data in X1-0
	movlw	LOW(ConfigData)	;Start of EE data
	addwf	PageOffset, W
	addwf	X1, W		;Add in register address
	addlw	0xFF		;Subtract 1 as registers start at 1
	movwf	AutoEEADR

	movf	X0, W
	call	StoreEE_12F 

	swapf	X1, W
	call	SendHex		;Echo back address
	movf	X1, W
	call	SendHex		
	movlw	"-"
	call	SendByte
	swapf	X0, W
	call	SendHex
	movf	X0, W
	call	SendHex		;Echo back address

	movlw	" "
	call	SendByte
	movlw	"W"
	call	SendByte
	movlw	"r"
	call	SendByte
	movlw	"i"
	call	SendByte
	movlw	"t"
	call	SendByte
	movlw	"t"
	call	SendByte
	movlw	"e"
	call	SendByte
	movlw	"n"
	call	SendByte
	call	CrLf

	goto	DoneCmd
;............
PCmd		;Select Page for programming 0 - 3 only
	movf	Stack2, W
	andlw	0x03
	movwf	PageOffset
	swapf	PageOffset	;Multiply by 16 (data is padded out to multiples of 16)

	movlw	" "
	call	SendByte
	movlw	"P"
	call	SendByte
	movlw	"a"
	call	SendByte
	movlw	"g"
	call	SendByte
	movlw	"e"
	call	SendByte
	movlw	" "
	call	SendByte
	swapf	PageOffset, W
	addlw	0x30
	call	SendByte
	call	CrLf

	call	ShowBlock

	goto    DoneCmd
;.............
;.............
DoneCmd
	call	ClearStack      ;Prevents spurious commands
	call	CrLf
    return
;------------------------------- 
Assemble232             ;Get here after Port B interrupt
    call    HalfBitDelay

 if RS232POL == 1
    btfss   SerIn     
 else
    btfsc   Serin
 endif

    goto    GetOut232   ;Invalid start bit

    movlw   d'8'        ;27 clocks after edge to here
    movwf   Counter
    clrf    RxChar
Rx232Loop
    call    HalfBitDelay
    call    HalfBitDelay
    bcf     STATUS , C

 if RS232POL == 1
    btfss   SerIn
 else
    btfsc   SerIn
 endif

    bsf     STATUS , C
    rrf     RxChar            
    decfsz  Counter
    goto    Rx232Loop   ;this loop 2 * HalfBitDelay + 9
    call    HalfBitDelay ;Part into stop bit - should be no more transitions


    movlw   "a"             
    subwf   RxChar, W       ;W = Char - "a"
    btfss   STATUS, C       ;if-ve (C=0) char < "a" so ignore
    goto    UpCaseDone

    movf    RxChar, W
    sublw   "z"             ;W = "z" - Char
    btfss   STATUS, C       ;if -ve (C=0)  char > "z" so ignore
    goto    UpCaseDone

    bcf     RxChar, 5       ;Reset bit 5 to force upper case
UpCaseDone
    movf    RxChar, W
    call    PushStack       ;RxChar >> Stack1, previous 5 chars move down one
    movf    RxChar , w      ;Returns with data in W as well as on stack
    
GetOut232
    return                  ;received byte in RxChar and W
;-------------------------
HalfBitDelay            ;1200 baud needs 833us, so delay 414 clocks here
    movlw   BAUDCONST   ; for a total of 414 * 2 + 8 = 833 clocks
	movwf   DelCount
BaudDelLoop             ;Total delay = 3.N + 5 = 414
	decfsz  DelCount
	goto    BaudDelLoop
	return
;-------------------------------
LongDelay
	movlw   d'150'      ;Approx 300ms delay
	movwf   Temp2
LongDelLoop1
	call	Delay
	call	Delay
	decfsz	Temp2
	goto	LongDelLoop1
	return
;------------------------------------
Delay                       ;1ms delay
    movlw   d'199'  
    movwf   DelCount
DelLoop
    nop
    nop
    decfsz  DelCount
    goto    DelLoop
    return
;------------------------------------
BCD2Bin		;Simple two digit one byte version for values up to 39
	movwf	Temp
	
	swapf	Temp, W
	andlw	0x0F		;Recover tens digit
	movwf	Temp2
	rlf	Temp2
	rlf	Temp2, W	
	andlw	0x3C
	movwf	Temp2		;Temp2 = Tens digit * 4
	swapf	Temp, W
	andlw	0x0F		;Recover tens digit
	addwf	Temp2		;Temp2 = Tens digit * 5
	bcf	STATUS, C
	rlf	Temp2		;Temp2 = Tens digit * 10

	movf	Temp, W
	andlw	0x0F		;Recover digits
	addwf	Temp2, W	;W = binary versino of BCD intry

	return
;------------------------------------
GetEE_12F                   ;Enter with address in W
    bsf     STATUS, RP0
    movwf   EEADR
    bsf     EECON1, RD
    movf    EEDATA, W
    bcf     STATUS, RP0
    return                  ;Exit with data in W
;-----------------------------------------
StoreEE_12F        ;Enter with W = EE data,  AutoEEADR = EE address
    bsf     STATUS, RP0      ;All EE addresses in bank 1
    movwf   EEDATA
    bcf     STATUS, RP0
    movf    AutoEEADR, W
    bsf     STATUS, RP0
    movwf   EEADR    

    bsf     EECON1 , WREN    ;Enable EEprom writing
    movlw   0x55
    movwf   EECON2
    movlw   0xAA
    movwf   EECON2
    bsf     EECON1 , WR

    bcf     STATUS , RP0
EEwaitW
    btfss   PIR1 , EEIF
    goto    EEwaitW

    bcf     PIR1 , EEIF      ;has to be cleared manually
    return
;-------------------------
PushStack
    movf    Stack9 , W
    movwf   StackA          
    movf    Stack8 , W
    movwf   Stack9
    movf    Stack7 , W
    movwf   Stack8              ;StackA contains oldest data
    movf    Stack6 , W
    movwf   Stack7
    movf    Stack5 , W
    movwf   Stack6          
    movf    Stack4 , W
    movwf   Stack5
    movf    Stack3 , W
    movwf   Stack4
    movf    Stack2 , W
    movwf   Stack3
    movf    Stack1 , W
    movwf   Stack2
    movf    RxChar , W
    movwf   Stack1
    return
;-------------------------
ClearStack
    movlw   0
    movwf   StackA
    movwf   Stack9
    movwf   Stack8
    movwf   Stack7
    movwf   Stack6
    movwf   Stack5
    movwf   Stack4
    movwf   Stack3
    movwf   Stack2
    movwf   Stack1
    return
;------------------------------------

IntroMsg
	movlw	"G"
	call	SendByte
	movlw	"4"
	call	SendByte
	movlw	"J"
	call	SendByte
	movlw	"N"
	call	SendByte
	movlw	"T"
	call	SendByte
	call	CrLf

	movlw	"L"
	call	SendByte
	movlw	"T"
	call	SendByte
	movlw	"C"
	call	SendByte
	movlw	"6"
	call	SendByte
	movlw	"9"
	call	SendByte
	movlw	"4"
	call	SendByte
	movlw	"6"
	call	SendByte
	movlw	" "
	call	SendByte

	movlw	"4"
	call	SendByte
	movlw	" "
	call	SendByte
	movlw	"F"
	call	SendByte
	movlw	"r"
	call	SendByte
	movlw	"e"
	call	SendByte
	movlw	"q"
	call	SendByte
	movlw	" "
	call	SendByte
	movlw	"C"
	call	SendByte
	movlw	"o"
	call	SendByte
	movlw	"n"
	call	SendByte
	movlw	"t"
	call	SendByte
	movlw	"r"
	call	SendByte
	movlw	"o"
	call	SendByte
	movlw	"l"
	call	SendByte
	call	CrLf

	movlw	"R"
	call	SendByte
	movlw	"x"
	call	SendByte	
	movlw	"x"
	call	SendByte	
	movlw	"x"
	call	SendByte	
	movlw	"x"
	call	SendByte	
	movlw	" "
	call	SendByte	
	movlw	"["
	call	SendByte	
	movlw	"C"
	call	SendByte	
	movlw	"R"
	call	SendByte	
	movlw	"]"
	call	SendByte	
	call	CrLf


	movlw	"["
	call	SendByte	
	movlw	"W"
	call	SendByte	
	movlw	"]"
	call	SendByte	
	movlw	"r"
	call	SendByte	
	movlw	"i"
	call	SendByte	
	movlw	"t"
	call	SendByte
	movlw	"e"
	call	SendByte

	movlw	" "
	call	SendByte
	movlw	"r"
	call	SendByte	
	movlw	"e"
	call	SendByte	
	movlw	"g"
	call	SendByte
	movlw	" "
	call	SendByte	
	movlw	"E"
	call	SendByte	
	movlw	"E"
	call	SendByte	

	call	CrLf

	movlw	"P"
	call	SendByte	
	movlw	"n"
	call	SendByte	
	movlw	" "
	call	SendByte	
	movlw	"P"
	call	SendByte	
	movlw	"a"
	call	SendByte	
	movlw	"g"
	call	SendByte	
	movlw	"e"
	call	SendByte	
	call	CrLf

	
	movlw	"E"
	call	SendByte	
	movlw	"X"
	call	SendByte	
	movlw	"I"
	call	SendByte	
	movlw	"T"
	call	SendByte	

	
	call	CrLf
	call	CrLf
	return		
;---------------------------------------
 
    cblock 0x20         ;64 Bytes available
        Wreg, StsReg
        Temp, Temp2
	Flags
	SaveBit
        BitCount
        Counter
	RegCount
	PageOffset
        DelCount
        AutoEEADR
        D0, D1
	X0, X1, X2, X3

        RxChar
        Stack1, Stack2, Stack3, Stack4, Stack5, Stack6, Stack7, Stack8, Stack9, StackA
    endc
;----------------------
	end

